/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.resource;

import java.io.*;
import java.util.*;

import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class BundleHelper {
    private ResourceBundle resourceBundle;

    public BundleHelper(String bundleName, Class clas) {
        try {
            ClassLoader loader = clas.getClassLoader();
            loader = fixLoader(loader);
            resourceBundle = ResourceBundle.getBundle(bundleName, Locale.getDefault(), loader);
        } catch (MissingResourceException e) {
            Log.warning("missing bundle", this, "BundleHelper", bundleName, clas);
        }
    }

    /* This is weird but apparently necessary.  On MS Windows the "getResource" method on the default ClassLoader
     * searches the jar directory.  On Unix and Mac is does not.  I'm not sure why it is only searched on 
     * Windows but this is certainly desirable behavior. 
     * 
     * We want the directory searched because we want to load the 
     * user's bundles from there.  For plugins this is handled by the custom subclass of URLClassLoader
     * that searches the plugin directory.  We cannot replace the system class loader but we can make up a fake class
     * loader that searches where we want to.  This classloader isn't actually used for anything other than loading
     * the resource bundle.  This seems really strange but does appear to work.
     */
    private ClassLoader fixLoader(ClassLoader loader) {
        if (loader == ClassLoader.getSystemClassLoader()) {
            File applicationDirectory = ApplicationProperties.getApplicationDirectory();
            loader = new DirectoryClassLoader(applicationDirectory, loader);
        }
        return loader;
    }

    public String getText(String text) {
        String s;
        if (resourceBundle == null) {
            Log.error("no resource bundle", BundleHelper.class, "getText", text);
            s = "Missing resource bundle for " + text;
        } else {
            try {
                s = resourceBundle.getString(text);
            } catch (MissingResourceException e) {
                Log.warning("missing resource", BundleHelper.class, "getText", text);
                s = "Missing resource for " + text;
            }
        }
        return s;
    }

    public String getText(String text, String macroReplacement) {
        String s = getText(text);
        s = StringUtilities.replace(s, "{0}", macroReplacement);
        return s;
    }

    public boolean isValid() {
        return resourceBundle != null;
    }

    public String toString() {
        return StringUtilities.getClassName(this);
    }

}
