/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.server;

import java.rmi.*;
import java.rmi.server.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.framestore.*;
import edu.stanford.smi.protege.model.query.*;

public class ServerFrameStore extends UnicastRemoteObject implements RemoteServerFrameStore {
    private FrameStore _delegate;
    private List _events = new ArrayList();
    private Map _sessionToRegistrationMap = new HashMap();
    private boolean _isDirty;

    public ServerFrameStore(FrameStore delegate, KnowledgeBase kb) throws RemoteException {
        _delegate = delegate;
        kb.setDispatchEventsEnabled(false);
        // kb.setJournalingEnabled(true);
    }

    private FrameStore getDelegate() {
        return _delegate;
    }

    public int getClsCount(RemoteSession session) {
        return getDelegate().getClsCount();
    }
    public int getSlotCount(RemoteSession session) {
        return getDelegate().getSlotCount();
    }
    public int getFacetCount(RemoteSession session) {
        return getDelegate().getFacetCount();
    }
    public int getSimpleInstanceCount(RemoteSession session) {
        return getDelegate().getSimpleInstanceCount();
    }
    public int getFrameCount(RemoteSession session) {
        return getDelegate().getFrameCount();
    }
    public synchronized List getDirectTemplateSlots(Cls cls, RemoteSession session) {
        return getDelegate().getDirectTemplateSlots(cls);
    }

    public synchronized void removeDirectTemplateSlot(Cls cls, Slot slot, RemoteSession session) {
        getDelegate().removeDirectTemplateSlot(cls, slot);
        markDirty();
    }

    public synchronized void moveDirectTemplateSlot(Cls cls, Slot slot, int index, RemoteSession session) {
        getDelegate().moveDirectTemplateSlot(cls, slot, index);
        markDirty();
    }

    public synchronized void addDirectSuperclass(Cls cls, Cls superclass, RemoteSession session) {
        getDelegate().addDirectSuperclass(cls, superclass);
        markDirty();
    }

    public synchronized void removeDirectSuperslot(Slot slot, Slot superslot, RemoteSession session) {
        getDelegate().removeDirectSuperslot(slot, superslot);
        markDirty();
    }

    public synchronized void removeDirectSuperclass(Cls cls, Cls superclass, RemoteSession session) {
        getDelegate().removeDirectSuperclass(cls, superclass);
        markDirty();
    }

    public synchronized void moveDirectSubclass(Cls cls, Cls subclass, int index, RemoteSession session) {
        getDelegate().moveDirectSubclass(cls, subclass, index);
        markDirty();
    }

    public synchronized List getDirectTemplateSlotValues(Cls cls, Slot slot, RemoteSession session) {
        return getDelegate().getDirectTemplateSlotValues(cls, slot);
    }

    public synchronized Set getSuperslots(Slot slot, RemoteSession session) {
        return getDelegate().getSuperslots(slot);
    }

    public synchronized Set getOwnSlots(Frame frame, RemoteSession session) {
        return getDelegate().getOwnSlots(frame);
    }

    public synchronized Set getInstances(Cls cls, RemoteSession session) {
        return getDelegate().getInstances(cls);
    }

    public synchronized Set getFramesWithDirectOwnSlotValue(Slot slot, Object value, RemoteSession session) {
        return getDelegate().getFramesWithDirectOwnSlotValue(slot, value);
    }

    public synchronized Set getClsesWithDirectTemplateSlotValue(Slot slot, Object value, RemoteSession session) {
        return getDelegate().getClsesWithDirectTemplateSlotValue(slot, value);
    }

    public synchronized Set getClsesWithDirectTemplateFacetValue(
        Slot slot,
        Facet facet,
        Object value,
        RemoteSession session) {
        return getDelegate().getClsesWithDirectTemplateFacetValue(slot, facet, value);
    }

    public synchronized Set getFramesWithMatchingDirectOwnSlotValue(
        Slot slot,
        String value,
        int maxMatches,
        RemoteSession session) {
        return getDelegate().getFramesWithMatchingDirectOwnSlotValue(slot, value, maxMatches);
    }

    public synchronized Set getClsesWithMatchingDirectTemplateFacetValue(
        Slot slot,
        Facet facet,
        String value,
        int maxMatches,
        RemoteSession session) {
        return getDelegate().getClsesWithMatchingDirectTemplateFacetValue(slot, facet, value, maxMatches);
    }

    public synchronized Set getClsesWithMatchingDirectTemplateSlotValue(
        Slot slot,
        String value,
        int maxMatches,
        RemoteSession session) {
        return getDelegate().getClsesWithMatchingDirectTemplateSlotValue(slot, value, maxMatches);
    }

    public synchronized List getDirectSuperclasses(Cls cls, RemoteSession session) {
        return getDelegate().getDirectSuperclasses(cls);
    }

    public synchronized Collection getTemplateSlotValues(Cls cls, Slot slot, RemoteSession session) {
        return getDelegate().getTemplateSlotValues(cls, slot);
    }

    public synchronized List getDirectTemplateFacetValues(Cls cls, Slot slot, Facet facet, RemoteSession session) {
        return getDelegate().getDirectTemplateFacetValues(cls, slot, facet);
    }

    public synchronized Set getClses(RemoteSession session) {
        return getDelegate().getClses();
    }

    public synchronized Set getTemplateFacets(Cls cls, Slot slot, RemoteSession session) {
        return getDelegate().getTemplateFacets(cls, slot);
    }

    public synchronized Frame getFrame(String name, RemoteSession session) {
        return getDelegate().getFrame(name);
    }

    public synchronized Frame getFrame(FrameID id, RemoteSession session) {
        return getDelegate().getFrame(id);
    }

    public synchronized Collection getOwnSlotValues(Frame frame, Slot slot, RemoteSession session) {
        return getDelegate().getOwnSlotValues(frame, slot);
    }

    public synchronized int getDirectOwnSlotValuesCount(Frame frame, Slot slot, RemoteSession session) {
        return getDelegate().getDirectOwnSlotValuesCount(frame, slot);
    }

    public synchronized List getDirectOwnSlotValues(Frame frame, Slot slot, RemoteSession session) {
        return getDelegate().getDirectOwnSlotValues(frame, slot);
    }

    public synchronized List getDirectInstances(Cls cls, RemoteSession session) {
        return getDelegate().getDirectInstances(cls);
    }

    public synchronized Set getSubclasses(Cls cls, RemoteSession session) {
        return getDelegate().getSubclasses(cls);
    }

    public synchronized Set getSlots(RemoteSession session) {
        return getDelegate().getSlots();
    }

    public synchronized Set getSuperclasses(Cls cls, RemoteSession session) {
        return getDelegate().getSuperclasses(cls);
    }

    public synchronized Set getSubslots(Slot slot, RemoteSession session) {
        return getDelegate().getSubslots(slot);
    }

    public synchronized void setDirectTemplateFacetValues(
        Cls cls,
        Slot slot,
        Facet facet,
        Collection values,
        RemoteSession session) {
        getDelegate().setDirectTemplateFacetValues(cls, slot, facet, values);
        markDirty();
    }

    public synchronized Facet createFacet(
        FrameID id,
        String name,
        Collection directTypes,
        boolean loadDefaults,
        RemoteSession session) {
        markDirty();
        return getDelegate().createFacet(id, name, directTypes, loadDefaults);
    }

    public synchronized List getDirectSubclasses(Cls cls, RemoteSession session) {
        return getDelegate().getDirectSubclasses(cls);
    }

    public synchronized Set getFrames(RemoteSession session) {
        return getDelegate().getFrames();
    }

    public synchronized void setDirectTemplateSlotValues(
        Cls cls,
        Slot slot,
        Collection values,
        RemoteSession session) {
        markDirty();
        getDelegate().setDirectTemplateSlotValues(cls, slot, values);
    }

    public synchronized Set getTypes(Instance instance, RemoteSession session) {
        return getDelegate().getTypes(instance);
    }

    public synchronized Set getTemplateSlots(Cls cls, RemoteSession session) {
        return getDelegate().getTemplateSlots(cls);
    }

    public synchronized Collection getTemplateFacetValues(Cls cls, Slot slot, Facet facet, RemoteSession session) {
        return getDelegate().getTemplateFacetValues(cls, slot, facet);
    }

    public synchronized void deleteCls(Cls cls, RemoteSession session) {
        getDelegate().deleteCls(cls);
        markDirty();
    }

    public synchronized void deleteSlot(Slot slot, RemoteSession session) {
        getDelegate().deleteSlot(slot);
        markDirty();
    }

    public synchronized void deleteFacet(Facet facet, RemoteSession session) {
        getDelegate().deleteFacet(facet);
        markDirty();
    }

    public synchronized void deleteSimpleInstance(SimpleInstance simpleInstance, RemoteSession session) {
        getDelegate().deleteSimpleInstance(simpleInstance);
        markDirty();
    }

    public synchronized Slot createSlot(
        FrameID id,
        String name,
        Collection directTypes,
        Collection directSuperslots,
        boolean loadDefaults,
        RemoteSession session) {
        markDirty();
        return getDelegate().createSlot(id, name, directTypes, directSuperslots, loadDefaults);
    }

    public synchronized List getDirectTypes(Instance instance, RemoteSession session) {
        return getDelegate().getDirectTypes(instance);
    }
    public synchronized List getDirectSubslots(Slot slot, RemoteSession session) {
        return getDelegate().getDirectSubslots(slot);
    }

    public synchronized void addDirectSuperslot(Slot slot, Slot superslot, RemoteSession session) {
        getDelegate().addDirectSuperslot(slot, superslot);
        markDirty();
    }

    public synchronized void addDirectTemplateSlot(Cls cls, Slot slot, RemoteSession session) {
        getDelegate().addDirectTemplateSlot(cls, slot);
        markDirty();
    }

    public synchronized void setDirectOwnSlotValues(Frame frame, Slot slot, Collection values, RemoteSession session) {
        getDelegate().setDirectOwnSlotValues(frame, slot, values);
        markDirty();
    }

    public synchronized Cls createCls(
        FrameID id,
        String name,
        Collection directTypes,
        Collection directSuperclasses,
        boolean loadDefaults,
        RemoteSession session) {
        markDirty();
        return getDelegate().createCls(id, name, directTypes, directSuperclasses, loadDefaults);
    }

    public synchronized Set getFacets(RemoteSession session) {
        return getDelegate().getFacets();
    }

    public synchronized Set executeQuery(Query query, RemoteSession session) {
        return getDelegate().executeQuery(query);
    }

    public synchronized void removeDirectType(Instance instance, Cls directType, RemoteSession session) {
        getDelegate().removeDirectType(instance, directType);
        markDirty();
    }
    public synchronized Set getReferences(Object value, RemoteSession session) {
        return getDelegate().getReferences(value);
    }

    public synchronized Set getMatchingReferences(String value, int maxMatches, RemoteSession session) {
        return getDelegate().getMatchingReferences(value, maxMatches);
    }

    public synchronized SimpleInstance createSimpleInstance(
        FrameID id,
        String name,
        Collection directTypes,
        boolean loadDefaults,
        RemoteSession session) {
        markDirty();
        return getDelegate().createSimpleInstance(id, name, directTypes, loadDefaults);
    }

    public synchronized void addDirectType(Instance instance, Cls type, RemoteSession session) {
        getDelegate().addDirectType(instance, type);
        markDirty();
    }

    public synchronized List getDirectSuperslots(Slot slot, RemoteSession session) {
        return getDelegate().getDirectSuperslots(slot);
    }

    public synchronized String getFrameName(Frame frame, RemoteSession session) {
        return getDelegate().getFrameName(frame);
    }

    public synchronized void setFrameName(Frame frame, String name, RemoteSession session) {
        getDelegate().setFrameName(frame, name);
        markDirty();
    }

    public synchronized Set getOwnFacets(Frame frame, Slot slot, RemoteSession session) {
        return getDelegate().getOwnFacets(frame, slot);
    }

    public synchronized Collection getOwnFacetValues(Frame frame, Slot slot, Facet facet, RemoteSession session) {
        return getDelegate().getOwnFacetValues(frame, slot, facet);
    }

    public synchronized Set getOverriddenTemplateSlots(Cls cls, RemoteSession session) {
        return getDelegate().getOverriddenTemplateSlots(cls);
    }

    public synchronized Set getDirectlyOverriddenTemplateSlots(Cls cls, RemoteSession session) {
        return getDelegate().getDirectlyOverriddenTemplateSlots(cls);
    }

    public synchronized Set getOverriddenTemplateFacets(Cls cls, Slot slot, RemoteSession session) {
        return getDelegate().getOverriddenTemplateFacets(cls, slot);
    }

    public synchronized Set getDirectlyOverriddenTemplateFacets(Cls cls, Slot slot, RemoteSession session) {
        return getDelegate().getOverriddenTemplateFacets(cls, slot);
    }

    public synchronized void removeDirectTemplateFacetOverrides(Cls cls, Slot slot, RemoteSession session) {
        getDelegate().removeDirectTemplateFacetOverrides(cls, slot);
        markDirty();
    }

    public synchronized void close(RemoteSession session) {
        // do nothing
    }

    public void register(RemoteSession session) {
        Registration registration = new Registration(_events.size());
        _sessionToRegistrationMap.put(session, registration);
    }

    public String toString() {
        return "ServerFrameStoreImpl";
    }

    public synchronized List getEvents(RemoteSession session) {
        List events;
        List newEvents = getDelegate().getEvents();
        if (session == null) {
            events = newEvents;
        } else {
            _events.addAll(newEvents);
            Registration reg = (Registration) _sessionToRegistrationMap.get(session);
            if (reg == null) {
                throw new IllegalStateException("Not registered");
            }
            int lastEvent = reg.getLastEvent();
            int size = _events.size();
            reg.setLastEvent(size);
            if (lastEvent == size) {
                events = Collections.EMPTY_LIST;
            } else {
                events = new ArrayList(_events.subList(lastEvent, size));
                // Log.trace(session + " events=" + events, this, "getEvents");
            }
        }
        // edu.stanford.smi.protege.util.Log.enter(this, "getEvents", session, events);
        return events;
    }

    public synchronized boolean beginTransaction(String name, RemoteSession session) {
        return getDelegate().beginTransaction(name);
    }

    public synchronized boolean commitTransaction(RemoteSession session) {
        return getDelegate().commitTransaction();
    }

    public synchronized boolean rollbackTransaction(RemoteSession session) {
        return getDelegate().rollbackTransaction();
    }

    public boolean isDirty() {
        return _isDirty;
    }

    private void markDirty() {
        _isDirty = true;
    }

    public void markClean() {
        _isDirty = false;
    }

    public List getDirectDomain(Slot slot, RemoteSession session) throws RemoteException {
        return getDelegate().getDirectDomain(slot);
    }

    public Set getDomain(Slot slot, RemoteSession session) throws RemoteException {
        return getDelegate().getDomain(slot);
    }

    public void moveDirectSubslot(Slot slot, Slot subslot, int index, RemoteSession session) throws RemoteException {
        getDelegate().moveDirectSubslot(slot, subslot, index);
    }

    public Set getFramesWithAnyDirectOwnSlotValue(Slot slot, RemoteSession session) throws RemoteException {
        return getDelegate().getFramesWithAnyDirectOwnSlotValue(slot);
    }

    public Set getClsesWithAnyDirectTemplateSlotValue(Slot slot, RemoteSession session) throws RemoteException {
        return getDelegate().getClsesWithAnyDirectTemplateSlotValue(slot);
    }

    public Set getDirectOwnSlotValuesClosure(Frame frame, Slot slot, RemoteSession session) throws RemoteException {
        return getDelegate().getDirectOwnSlotValuesClosure(frame, slot);
    }

}
