/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.server;

import java.awt.*;
import java.rmi.*;

import javax.swing.*;

import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class ServerPanel extends JPanel implements Validatable {
    private static String _lastUserName = "Guest";
    private static String _lastHostName = "localhost";
    private static String _lastPassword = "guest";
    private JTextField _usernameField;
    private JTextField _passwordField;
    private JTextField _hostNameField;
    private RemoteServer _server;
    private RemoteSession _session;

    public ServerPanel() {
        _usernameField = ComponentFactory.createTextField();
        _usernameField.setText(_lastUserName);
        _passwordField = ComponentFactory.createPasswordField();
        _passwordField.setText(_lastPassword);
        _hostNameField = ComponentFactory.createTextField();
        _hostNameField.setText(_lastHostName);

        setLayout(new GridLayout(3, 0));
        add(new LabeledComponent("User Name", _usernameField));
        add(new LabeledComponent("Password", _passwordField));
        add(new LabeledComponent("Host Machine Name", _hostNameField));
    }

    public boolean validateContents() {
        boolean isValid = false;
        try {
            isValid = isValidConfiguration();
        } catch (Exception e) {
            // do nothing
        }
        return isValid;
    }

    private boolean isValidConfiguration() {
        boolean isValid = false;
        String serverName = _hostNameField.getText();
        _server = connectToHost(serverName);
        if (_server == null) {
            ModalDialog.showMessageDialog(this, "Unable to connect to server");
        } else {
            String username = _usernameField.getText();
            String password = _passwordField.getText();
            _session = createSession(username, password);
            if (_session == null) {
                String text = "The connection failed either because:\n";
                text += "    Invalid username/password combination\n";
                text += "    A firewall problem";
                ModalDialog.showMessageDialog(this, text, "Connection Failed");
            } else {
                isValid = true;
            }
        }
        return isValid;
    }

    public void saveContents() {
        _lastUserName = _usernameField.getText();
        _lastHostName = _hostNameField.getText();
        _lastPassword = _passwordField.getText();
    }

    public RemoteServer getServer() {
        return _server;
    }

    public RemoteSession getSession() {
        return _session;
    }

    private static RemoteServer connectToHost(String serverName) {
        RemoteServer server = null;
        try {
            server = (RemoteServer) Naming.lookup("//" + serverName + "/" + Server.getBoundName());
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing
        }
        return server;
    }

    private RemoteSession createSession(String username, String password) {
        RemoteSession session = null;
        try {
            session = _server.openSession(username, SystemUtilities.getMachineName(), password);
        } catch (RemoteException e) {
            e.printStackTrace();
            // do nothing
        }
        return session;
    }
}