/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.util;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

/**
 * A list box that is specialized to work with file names.  It allows the user to browse for names using the standard
 * file browser.
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class URIList extends AbstractSelectableComponent {
    private JList _uriList;
    private String _description;
    private String _extension;

    public URIList(String label, Collection uris, String extension, String description) {
        _description = description;
        _extension = extension;
        setLayout(new BorderLayout());
        LabeledComponent c = new LabeledComponent(label, createComponent(uris));
        c.addHeaderButton(newAddAction());
        c.addHeaderButton(newRemoveAction());
        add(c);
        setPreferredSize(new Dimension(300, 200));
    }

    private void browse() {
        JFileChooser chooser = ComponentFactory.createFileChooser(_description, _extension);
        chooser.setMultiSelectionEnabled(true);
        int openDialogResult = chooser.showOpenDialog(this);
        switch (openDialogResult) {
            case JFileChooser.ERROR_OPTION :
                // Get this on 'close"
                break;
            case JFileChooser.CANCEL_OPTION :
                break;
            case JFileChooser.APPROVE_OPTION :
                File[] files = chooser.getSelectedFiles();
                if (files != null) {
                    for (int i = 0; i < files.length; ++i) {
                        ComponentUtilities.addSelectedListValue(_uriList, files[i].toURI());
                    }
                }
                break;
            default :
                Assert.fail("bad result: " + openDialogResult);
        }
    }

    public void clearSelection() {
        _uriList.clearSelection();
    }

    private JComponent createComponent(Collection files) {
        _uriList = ComponentFactory.createList(newAddAction());
        _uriList.addListSelectionListener(new ListSelectionListenerAdapter(this));
        if (files != null) {
            ComponentUtilities.setListValues(_uriList, files);
        }
        return new JScrollPane(_uriList);
    }

    public Collection getURIs() {
        return ComponentUtilities.getListValues(_uriList);
    }

    public Collection getSelection() {
        return ComponentUtilities.getSelection(_uriList);
    }

    private Action newAddAction() {
        return new AddAction("Browse for File") {
            public void onAdd() {
                browse();
            }
        };
    }

    private Action newRemoveAction() {
        return new RemoveAction("Remove Uri", this) {
            public void onRemove(Collection values) {
                ComponentUtilities.removeListValues(_uriList, values);
            }
        };
    }
}
